#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::fs - Filesystem type control module
#
package Security::fs;

use Sys::Filesystem;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( init );

sub init {
	print ( STDERR  "In Security::fs::init()\n") if ( $main::opts{'d'} );

	&main::read_config('ignorefs');

	# No longer need to register a wanted subroutine, since we are just
	# going to add the filesystems of our excluded types to the ignoredir
	# configuration option (see Security::find.pm).

	# if ignoredir is not defined, assume we have not yet called
	# Security::find::init(), and fail, otherwise it will overwrite
	# our adding directories to ignoredir
	&main::security_die ("Security::fs::init() called before Security::find::init()\nPlease fix module load ordering\n")
		if ( ! defined @main::ignoredir );

	my $fs = new Sys::Filesystem;
	my @filesystems = $fs->filesystems();
	for (@filesystems) {
		for (my ($i) = 0; $i < scalar @main::ignorefs; $i++) {
			if ( $fs->format($_) eq $main::ignorefs[$i] ) {
				my $dir = $_;
				$dir =~ s/\\040/ /g if ( $dir =~ m/\\040/ );
				push (@main::ignoredir, $dir);
				printf STDERR "\tAdding $dir to ignoredir because it is of fstyp ($main::ignorefs[$i])\n" 
					if ( $main::opts{'d'} );
			}
		}
	}
}

1;

__END__

=head1 NAME

Security::fs - File system type module

=head1 SYNOPSIS

Security::fs::check()

=head1 DESCRIPTION

Will prune filesystems based on a list of filesystem types not to
traverse.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Checks to see if L<Security::find::init()> has been called already,
so any values put into C<@main::ignoredir> do not get lost.

Calls C<read_config (ignorefs)> to get the list of filesystem types
to prune out of our filesystem traversal.

Calls L<Sys::Filesystem::filesystems> to get the list of filesystems
that are currently mounted.  Matches up the filesystem type with the
list of filesystems to ignore in C<@main::ignorefs>.  If a there is
a match, it will put the mountpoint on the C<@main::ignoredir> list.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorefs>

The list of filesystem types to ignore during the filesystem traversal.

=back

=head1 SEE ALSO

L<Sys::Filesystem>

=cut

