#!/usr/bin/env perl
#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  security.pl - System Security Script
#

use Getopt::Std;
use File::Basename;
use Sys::Hostname;
use POSIX;
use Net::SMTP;
use Security::util::SHA1;
use Security::util::users;
use Security::util::groups;
use strict;
use warnings;
no warnings 'once';

our $VERSION = '3.4.7';
our ($smtp, $title, @modlist, @mailhost, $format, @load, %opts, $fh);
our $OWNER = 0;
our $GROUP = 0;
our $file_open = 0;
our $host = hostname();
$opts{'C'} = '##SITEPREFIX##/etc/security/security.cf';
our $checksum_file = '##SITEPREFIX##/var/db/security/checksum.db';

sub usage {
	my ($opts) = @_;
	my $PROGNAME = File::Basename::basename($0);

	print "Usage: $PROGNAME -[$opts]\n";
	print "\t-C <conf file>\tUse <conf file> instead of default config file.\n";
	print "\t-d\t\tDebug mode\n";
	print "\t-h\t\tThis help screen\n";
	print "\t-M <addr>\tMail report to <addr>, defaults to 'root\@$host'\n";
	print "\t\t\tif <addr> starts with a /, write to a file.\n";
	print "\t-V\t\tPrint the program version ($VERSION)\n";
	exit 0;
}

sub init {
	print ( STDERR "In init\n") if ( $opts{'d'} );

	&report_header();

	# Get the checksum format, sets $format
	&Security::util::SHA1::init();

	# Load up the password file information.
	&Security::util::users::init();

	# Load up the group file information.
	&Security::util::groups::init();

	&check_conf_files();
	&read_config('load');
	@load = ( "OS::$^O", @load );		# Get the OS specific module loaded too
	&load_modules();
}

sub check_conf_files {
	print ( STDERR "In check_conf_files\n") if ( $opts{'d'} );

	# Now check if the configfile is ok.
	&check_conf_file ($opts{'C'}, 00400, $OWNER, $GROUP, 'config file');

	# Check if this script itself is ok
	&check_conf_file ($0, 00700, $OWNER, $GROUP, 'security script');

	# Check if the checksum db is ok
	&check_conf_file ($checksum_file, 00600, $OWNER, $GROUP, 'checksum db');
}

sub read_config {
	my ($field) = @_;
	my $FP;

	no strict;
	print ( STDERR "In read_config($field)\n") if ( $opts{'d'} );
	if (open ($FP, $opts{'C'})) {
		while (<$FP>) {
			chop;
			@{$field} = split (/[ \t]+/o, substr ($_, index ($_, ' ')+1))
				if (/^$field /);
		}
		close $FP;
	} else {
		# Use STDOUT for the errors since we can't connect (might be
		# reading mailhost)
		$opts{'M'} = '-';
		&security_die("Could not read config file ($opts{'C'})\n");
	}

	if ( $opts{'d'} ) {
		for (my ($i) = 0; $i <= $#{$field}; $i++) {
			printf STDERR "\t$field = ${$field}[$i]\n";
		}
	}
	use strict;
}

sub load_modules {
	print ( STDERR "In load_modules\n") if ( $opts{'d'} );

	# Now load all the requested modules from the config file.
	for (my ($i) = 0; $i <= $#load; $i++) {
		printf STDERR "\tLoading Security::$load[$i]\n" if ($opts{'d'});
		my ($pkg) = "use Security::$load[$i]";
		eval $pkg;
		&security_die("Error loading module Security::$load[$i] => $@\n") if ( $@ );
	}
}

sub report_header {
	my ($date);

	# Mail report information
	$date = strftime ( "%m/%d/%Y", localtime(time()));
	$title = "$host security ($VERSION) report for $date";
}

sub parse_options {
	my $opts = 'C:dhM:V';
	&getopts($opts, \%opts);

	&usage($opts) if ( $opts{'h'} );

	if ( $opts{'V'} ) {
		print $VERSION . "\n";
		exit 0;
	}
}

sub call {
	my ($func) = @_;
	my ($cmd);
	printf STDERR "In call($func)\n" if ( $opts{'d'} );

	for (my ($i) = 0; $i <= $#load; $i++) {
		$cmd = "Security::$load[$i]::$func";
		no strict;
		&{$cmd} if ( defined &{$cmd} );
		use strict;
	}
}

sub create_secure_file {
	my ($file, $uid, $gid, $mode) = @_;
	my ($FP);

	umask 077;
	$( = $uid; $) = $gid;
	open ($FP, "> $file") || &security_die("Could not open secure file\n");
	close $FP;
	chown ( $uid, $gid, $file);
	chmod ( $mode, $file);
}

sub check_conf_file {
	my ($file, $wanted_mode, $wanted_uid, $wanted_gid, $msg) = @_;
	print ( STDERR "In check_conf_file($file, ...)\n") if ( $opts{'d'} );
	my ($cursum);

    	my($dev,$ino,$mode,$nlink,$uid,$gid) = lstat($file);

	&security_die("Could not find file $file\n") 
		if ( ! ( defined $dev && defined $ino && defined $mode && defined $nlink && defined $uid && defined $gid )); 

	printf STDERR "\t%s mode = %o, uid = %d, gid = %d\n", 
		$file, ($mode & 00777), $uid, $gid if ( $opts{'d'} );

	&security_die("Permissions, owner or group not correct on $msg\n") 
		if (($mode & 00777) != $wanted_mode || 
			$uid != $wanted_uid || 
			$gid != $GROUP );
}

sub check_fp {
	$opts{'M'} = "root\@$host" if ( ! defined $opts{'M'} );

	if ( !$file_open ) {
		if ( $opts{'M'} eq '-' ) {
			$fh = \*STDOUT;
			print ($fh "$title\n");
			$file_open = 1;
		} elsif ( $opts{'M'} =~ /^\//o ) {
			&create_secure_file ($opts{'M'}, $OWNER, $GROUP, 00600);
			open ($fh, ">$opts{'M'}" ) 
				|| &security_die("Could not open $opts{'M'}\n");
			print ($fh "$title\n");
			$file_open = 1;
		} else { 
			&read_config('mailhost');
			if (!($smtp = Net::SMTP->new($mailhost[0]))) {

				# Use STDOUT for the errors since we can't connect
				$opts{'M'} = '-';
			 	&security_die("Could not connect to $mailhost[0]\n");
			}
			$smtp->mail("root\@$host");
			$smtp->to($opts{'M'});
			$smtp->data();
			$smtp->datasend("To: $opts{'M'}\n");
			$smtp->datasend("From: root\@$host\n");
			$smtp->datasend("Subject: $title\n");
			$smtp->datasend("\n");

			$file_open = 2;
		}
	}
}

sub print_header {
	my ($header) = @_;

	&check_fp();
	if ($file_open == 1) {
		print ($fh $header);
	} else {
		$smtp->datasend($header);
	}
}

sub _format_values {
	my ($field) = @_;
	my ($res) = '';

	if ( ref $field eq 'HASH' ) {
		$res = ' ' . join (' ', %$field);
	} elsif ( ref $field eq 'ARRAY' ) {
		$res = ' ' . join (' ', @$field);
	} elsif ( $field ne '1' ) {
		$res = qq{ $field};
	}
	return $res;
}

sub print_data {
	print ( STDERR "In print_data\n") if ( $opts{'d'} );
	my ($header,%ary) = @_;
	my ($cnt) = 0;

	foreach my $i (sort keys %ary) {
		if (! $cnt ) {
			&print_header($header);
			$cnt=1;
		}
		if ( $file_open == 1) {
			print ($fh $i . _format_values($ary{$i}) . "\n");
		} else {
			$smtp->datasend($i . _format_values($ary{$i}) . "\n");
		}
	}
}

# Replacement function for the die perl call, if the email address option
# is set, write to that instead of STDERR.
sub security_die {
	my ($msg) = @_;

	&check_fp();
	if ( $opts{'M'} eq '-' ) {
		die $msg;
	} elsif ($file_open == 1) {
		print ($fh $msg);
	} else {
		$smtp->datasend($msg);
		$smtp->dataend();
		$smtp->quit;
	}
	exit 1;
}

# Main.
&parse_options();
&init();
&call('init');
&call('check');
&call('print');
&call('destroy');
if ($file_open == 2) {
	$smtp->dataend();
	$smtp->quit;
}

exit(0);

__END__

=head1 NAME

security.pl - File system security and verification program

=head1 SYNOPSIS

security.pl -[C:dhM:]

=head1 DESCRIPTION

A perl script to check for inconsistant, incorrect configurations that may
be a sign of a security issue.  Checks for stuff like files assigned a UID
that has no associated password entry, multiple UID 0 accounts, checksums
changing on set UID/GID files, etc.  Results are mailed to root or written
to a file on the system.

The script supports the following command line switches:

=over 4

=item I<-C conf file>

Use I<conf file> instead of default config file.

=item I<-d>

Display debugging trace output on STDERR.

=item I<-h>

Display all the command line switches the program supports.

=item I<-M email address | absolute file path>

Mail problem report to I<email address>.  If I<absoulte file path> is given,
write output to that file.  Defaults to mailing to I<root@$HOSTNAME>

=item I<-V>

Print version number and exit.

=back

=head1 API

The security program will support the following API calls for each module. 

=over 4

=item I<init()>

A routine to initalize any variables that will be needed later on.  Good time
to call I<read_config> (see below).  All of the OS specific modules will set
the default locations for system commands in this routine.

=item I<check()>

A routine to run our check of the system for any problems.  Storing results
in a module local variable to be displayed later.

=item I<print()>

A routine to take the values stored in the module local variable and format
the results.  Good time to call I<print_data>.

=item I<destroy()>

A routine to clean up any variables or commands that we might want to save
for a future run.

=back

It also provides the following helper API calls which can be used in a module.

=over 4

=item I<read_config ($variable)>

A routine to read $variable from the config file and store the values
into a global variable of the same name.

=item I<print_data ($header, %hash)>

A routine to write out any configuration errors found.  If %hash has any
keys defined, it will write out $header to the output format, followed by
each key (and optionally the value stored at the key, if the value is not
equal to F<1> (numeric one).

=item I<security_die ($msg)>

A routine that will write out any I<die> type message to the output
format defined by the F<-M> option on the command line.  This routine
will exit the program with a non zero return code.

=back

B<NOTE>, other modules may provide other helper API calls.  See C<Security::db> and C<Security::find> for examples.

And the script provides the following global variables which can be accessed by
the module.

=over 4

=item I<%main::opts>

The list of command line options stored in a hash.  Most common usage is
I<$main::opts{'d'}>, the debug flag.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<load>

The list of modules to dynamically load.  The script will automatically add
C<$^O> to the end of list.

=item C<mailhost>

This variable is SMTP host the mail should be sent to.  Defaults to
C<localhost>, but may not work on hosts that does not have a STMP agent
installed or running.

=back

=head1 HISTORY

This security script came out of enhancements to the C<FreeBSD> F</etc/security>
script which was in version 2.1.  After reading the C<HP-UX System Security
Manual> I felt there needed to be more checks.  After being rebuffed with 
some my suggestions, I decided to write and maintain my own script.

=head1 FILES

=over 4

=item security.cf

The configuration file for this script.

=item checksum.db

The list of SHA1 checksums for all files we are monitoring.  Stored in a
DBM file format.

=item Security::*.pm

All the plugin modules.

=back

=head1 SEE ALSO

L<Security::*>

L<aide(1)>

L<cisecurity.org>

L<tiger(8)>

L<tripwire(8)>

L<perl(1)>

L<FreeBSD PR 1538>

=cut
