#!/usr/bin/env perl
#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  04.t - Test the find function and associated tests.
#

use Test;
use strict;
use Unix::Mknod qw(:all);
use Fcntl qw(:mode);
use File::stat;
use POSIX qw(getcwd);

plan tests => 13;

my (@FILES, $uid, $gid);
our ($format, %opts, @sharedregex, $kerneldir);
#$opts{'d'} = 1;

use Security::util::users;
use Security::util::groups;
use Security::find;
use Security::fs;
use Security::devs;
use Security::kernel;
use Security::library;
use Security::owner;
use Security::suid;
my $os_module = "Security::OS::$^O";
eval "use $os_module";

# Override the find search root point
my $cwd = getcwd;
$Security::find::root = $cwd;
$sharedregex[0] = 'lib[^/]*\.so(\..*)?$';
our @ignoredir = ("$cwd/blib");

sub read_config {}
sub print_data {}

sub create_file {
	my ($file, $uid, $gid, $mode) = @_;
	my ($FP);

	open ($FP, "> $file") || die "Could not create test file ($file)\n";
	close $FP;
	if ( $< eq 0 ) {
		chown $uid, $gid, $file || die "Could not chown file ($file)\n";
	} else {
		chown -1, $gid, $file || die "Could not chgrp file ($file)\n";
	}
	chmod $mode, $file if ( defined $mode );
	push @FILES, $file;
}

&Security::util::users::init();
&Security::util::SHA1::init();
&Security::util::groups::init();
my $func = $os_module . '::init';
no strict;
&{$func}();
use strict;
&Security::find::init();
&Security::fs::init();
&Security::devs::init();
&Security::kernel::init();
&Security::library::init();
&Security::owner::init();
&Security::suid::init();

# Create files that we should catch
$kerneldir = "$cwd/t";
if ( $> eq 0 ) {
	my ($st)=stat('/dev/null');
	mknod("$cwd/t/null", S_IFCHR|0666, makedev(major($st->rdev),minor($st->rdev)));
	push @FILES, "$cwd/t/null";
}

&create_file ("$cwd/t/libfoo.so", 0, 0, 0752);	# World writable shared library

# Search for an unassigned UID
my $name = 'foo';
for ($uid = 0; $uid < 65532 && defined $name; $uid++) {
	$name = &Security::util::users::get('uid', $uid, 'name');
}
$uid--;

# Search for an unassigned GID
$name = 'foo';
for ($gid = 0; $gid < 65532 && defined $name; $gid++) {
	$name = &Security::util::groups::get('gid', $gid, 'name');
}
$gid--;

&create_file ("$cwd/t/unowned", $uid, 0, 0644);
&create_file ("$cwd/t/nogroup", 0, $gid, 0644);

&create_file ("$cwd/t/suid", 0, 0, 04752);		# World writable suid script

# Now run our find....
&Security::find::check();

# Now do our tests
ok ( scalar keys %Security::kernel::kernelperms, 2, "Found not right number of kernel permission issues");
ok ( scalar keys %Security::library::libraryperms, 1, "Found not right number of library permission issues");
skip (
	($< != 0? "Test needs to be run as root": 0),
	scalar keys %Security::devs::devs, 1,
);
skip (
	($< != 0? "Test needs to be run as root": 0),
	scalar keys %Security::owner::nouids, 1,
);
skip (
	($< != 0? "Test needs to be run as root": 0),
	scalar keys %Security::owner::nogids, 1,
);
ok ( scalar keys %Security::suid::suidperms, 1, "Found not right number of suid permission issues");

ok ( exists $Security::kernel::kernelperms{"$cwd/t/suid"}, 1, "Expected to find world writable file $cwd/t/suid");
ok ( exists $Security::kernel::kernelperms{"$cwd/t/libfoo.so"}, 1, "Expected to find world writable file $cwd/t/libfoo.so");
ok ( exists $Security::library::libraryperms{"$cwd/t/libfoo.so"}, 1, "Expected to find world writable shared library");
skip (
	($< != 0? "Test needs to be run as root": 0),
	exists $Security::devs::devs{"$cwd/t/null"},
);
skip (
	($< != 0? "Test needs to be run as root": 0),
	exists $Security::owner::nouids{"$cwd/t/unowned"},
);
skip (
	($< != 0? "Test needs to be run as root": 0),
	exists $Security::owner::nogids{"$cwd/t/nogroup"},
);
ok ( exists $Security::suid::suidperms{"$cwd/t/suid"}, 1, "Expected to find SUID world writable file");

unlink (@FILES);

exit 0;
